@@ -3,29 +3,29 @@ use crate::{
     cmdbar::CommandBar,
     components::{
         event_pump, CommandBlocking, CommandInfo, CommitComponent,
-        Component, DrawableComponent, HelpComponent,
-        InspectCommitComponent, MsgComponent, ResetComponent,
-        StashMsgComponent,
+        Component, DrawableComponent, ExternalEditorComponent,
+        HelpComponent, InspectCommitComponent, MsgComponent,
+        ResetComponent, StashMsgComponent,
     },
+    input::{Input, InputEvent, InputState},
     keys,
     queue::{Action, InternalEvent, NeedsUpdate, Queue},
-    strings,
+    strings::{self, commands, order},
     tabs::{Revlog, StashList, Stashing, Status},
-    ui::style::Theme,
+    ui::style::{SharedTheme, Theme},
 };
 use anyhow::{anyhow, Result};
 use asyncgit::{sync, AsyncNotification, CWD};
 use crossbeam_channel::Sender;
 use crossterm::event::{Event, KeyEvent};
-use strings::{commands, order};
+use std::{cell::Cell, cell::RefCell, rc::Rc};
 use tui::{
     backend::Backend,
-    layout::{Constraint, Direction, Layout, Rect},
-    style::Modifier,
-    style::Style,
+    layout::{Constraint, Direction, Layout, Margin, Rect},
     widgets::{Block, Borders, Tabs},
     Frame,
 };
+
 ///
 pub struct App {
     do_quit: bool,
@@ -35,56 +35,75 @@ pub struct App {
     commit: CommitComponent,
     stashmsg_popup: StashMsgComponent,
     inspect_commit_popup: InspectCommitComponent,
-    cmdbar: CommandBar,
+    external_editor_popup: ExternalEditorComponent,
+    cmdbar: RefCell<CommandBar>,
     tab: usize,
     revlog: Revlog,
     status_tab: Status,
     stashing_tab: Stashing,
     stashlist_tab: StashList,
     queue: Queue,
-    theme: Theme,
+    theme: SharedTheme,
+    input: Input,
+
+    // "Flags"
+    requires_redraw: Cell<bool>,
 }
 
 // public interface
 impl App {
     ///
-    pub fn new(sender: &Sender<AsyncNotification>) -> Self {
+    pub fn new(
+        sender: &Sender<AsyncNotification>,
+        input: Input,
+    ) -> Self {
         let queue = Queue::default();
 
-        let theme = Theme::init();
+        let theme = Rc::new(Theme::init());
 
         Self {
-            reset: ResetComponent::new(queue.clone(), &theme),
-            commit: CommitComponent::new(queue.clone(), &theme),
+            input,
+            reset: ResetComponent::new(queue.clone(), theme.clone()),
+            commit: CommitComponent::new(
+                queue.clone(),
+                theme.clone(),
+            ),
             stashmsg_popup: StashMsgComponent::new(
                 queue.clone(),
-                &theme,
+                theme.clone(),
             ),
             inspect_commit_popup: InspectCommitComponent::new(
-                &queue, sender, &theme,
+                &queue,
+                sender,
+                theme.clone(),
+            ),
+            external_editor_popup: ExternalEditorComponent::new(
+                theme.clone(),
             ),
             do_quit: false,
-            cmdbar: CommandBar::new(&theme),
-            help: HelpComponent::new(&theme),
-            msg: MsgComponent::new(&theme),
+            cmdbar: RefCell::new(CommandBar::new(theme.clone())),
+            help: HelpComponent::new(theme.clone()),
+            msg: MsgComponent::new(theme.clone()),
             tab: 0,
-            revlog: Revlog::new(&queue, sender, &theme),
-            status_tab: Status::new(sender, &queue, &theme),
-            stashing_tab: Stashing::new(sender, &queue, &theme),
-            stashlist_tab: StashList::new(&queue, &theme),
+            revlog: Revlog::new(&queue, sender, theme.clone()),
+            status_tab: Status::new(&queue, sender, theme.clone()),
+            stashing_tab: Stashing::new(
+                sender,
+                &queue,
+                theme.clone(),
+            ),
+            stashlist_tab: StashList::new(&queue, theme.clone()),
             queue,
             theme,
+            requires_redraw: Cell::new(false),
         }
     }
 
     ///
-    pub fn draw<B: Backend>(
-        &mut self,
-        f: &mut Frame<B>,
-    ) -> Result<()> {
+    pub fn draw<B: Backend>(&self, f: &mut Frame<B>) -> Result<()> {
         let fsize = f.size();
 
-        self.cmdbar.refresh_width(fsize.width);
+        self.cmdbar.borrow_mut().refresh_width(fsize.width);
 
         let chunks_main = Layout::default()
             .direction(Direction::Vertical)
@@ -92,13 +111,13 @@ impl App {
                 [
                     Constraint::Length(2),
                     Constraint::Min(2),
-                    Constraint::Length(self.cmdbar.height()),
+                    Constraint::Length(self.cmdbar.borrow().height()),
                 ]
                 .as_ref(),
             )
             .split(fsize);
 
-        self.cmdbar.draw(f, chunks_main[2]);
+        self.cmdbar.borrow().draw(f, chunks_main[2]);
 
         self.draw_tabs(f, chunks_main[0]);
 
@@ -117,61 +136,75 @@ impl App {
     }
 
     ///
-    pub fn event(&mut self, ev: Event) -> Result<()> {
+    pub fn event(&mut self, ev: InputEvent) -> Result<()> {
         log::trace!("event: {:?}", ev);
 
-        if self.check_quit_key(ev) {
-            return Ok(());
-        }
+        if let InputEvent::Input(ev) = ev {
+            if self.check_quit_key(ev) {
+                return Ok(());
+            }
 
-        let mut flags = NeedsUpdate::empty();
+            let mut flags = NeedsUpdate::empty();
 
-        if event_pump(ev, self.components_mut().as_mut_slice())? {
-            flags.insert(NeedsUpdate::COMMANDS);
-        } else if let Event::Key(k) = ev {
-            let new_flags = match k {
-                keys::TAB_TOGGLE => {
-                    self.toggle_tabs(false)?;
-                    NeedsUpdate::COMMANDS
-                }
-                keys::TAB_TOGGLE_REVERSE
-                | keys::TAB_TOGGLE_REVERSE_WINDOWS => {
-                    self.toggle_tabs(true)?;
-                    NeedsUpdate::COMMANDS
-                }
-                keys::TAB_1
-                | keys::TAB_2
-                | keys::TAB_3
-                | keys::TAB_4 => {
-                    self.switch_tab(k)?;
-                    NeedsUpdate::COMMANDS
-                }
+            if event_pump(ev, self.components_mut().as_mut_slice())? {
+                flags.insert(NeedsUpdate::COMMANDS);
+            } else if let Event::Key(k) = ev {
+                let new_flags = match k {
+                    keys::TAB_TOGGLE => {
+                        self.toggle_tabs(false)?;
+                        NeedsUpdate::COMMANDS
+                    }
+                    keys::TAB_TOGGLE_REVERSE
+                    | keys::TAB_TOGGLE_REVERSE_WINDOWS => {
+                        self.toggle_tabs(true)?;
+                        NeedsUpdate::COMMANDS
+                    }
+                    keys::TAB_1
+                    | keys::TAB_2
+                    | keys::TAB_3
+                    | keys::TAB_4 => {
+                        self.switch_tab(k)?;
+                        NeedsUpdate::COMMANDS
+                    }
 
-                keys::CMD_BAR_TOGGLE => {
-                    self.cmdbar.toggle_more();
-                    NeedsUpdate::empty()
-                }
+                    keys::CMD_BAR_TOGGLE => {
+                        self.cmdbar.borrow_mut().toggle_more();
+                        NeedsUpdate::empty()
+                    }
 
-                _ => NeedsUpdate::empty(),
-            };
+                    _ => NeedsUpdate::empty(),
+                };
 
+                flags.insert(new_flags);
+            }
+
+            let new_flags = self.process_queue()?;
             flags.insert(new_flags);
-        }
 
-        let new_flags = self.process_queue()?;
-        flags.insert(new_flags);
-
-        if flags.contains(NeedsUpdate::ALL) {
-            self.update()?;
-        }
-        //TODO: make this a queue event?
-        //NOTE: set when any tree component changed selection
-        if flags.contains(NeedsUpdate::DIFF) {
-            self.status_tab.update_diff()?;
-            self.inspect_commit_popup.update_diff()?;
-        }
-        if flags.contains(NeedsUpdate::COMMANDS) {
-            self.update_commands();
+            if flags.contains(NeedsUpdate::ALL) {
+                self.update()?;
+            }
+            //TODO: make this a queue event?
+            //NOTE: set when any tree component changed selection
+            if flags.contains(NeedsUpdate::DIFF) {
+                self.status_tab.update_diff()?;
+                self.inspect_commit_popup.update_diff()?;
+            }
+            if flags.contains(NeedsUpdate::COMMANDS) {
+                self.update_commands();
+            }
+        } else if let InputEvent::State(polling_state) = ev {
+            self.external_editor_popup.hide();
+            if let InputState::Paused = polling_state {
+                if let Err(e) = self.commit.show_editor() {
+                    let msg =
+                        format!("failed to launch editor:\n{}", e);
+                    log::error!("{}", msg.as_str());
+                    self.msg.show_msg(msg.as_str())?;
+                }
+                self.requires_redraw.set(true);
+                self.input.set_polling(true);
+            }
         }
 
         Ok(())
@@ -181,11 +214,14 @@ impl App {
     /// forward ticking to components that require it
     pub fn update(&mut self) -> Result<()> {
         log::trace!("update");
+
         self.status_tab.update()?;
         self.revlog.update()?;
         self.stashing_tab.update()?;
         self.stashlist_tab.update()?;
 
+        self.update_commands();
+
         Ok(())
     }
 
@@ -219,6 +255,17 @@ impl App {
             || self.revlog.any_work_pending()
             || self.stashing_tab.anything_pending()
             || self.inspect_commit_popup.any_work_pending()
+            || self.input.is_state_changing()
+    }
+
+    ///
+    pub fn requires_redraw(&self) -> bool {
+        if self.requires_redraw.get() {
+            self.requires_redraw.set(false);
+            true
+        } else {
+            false
+        }
     }
 }
 
@@ -232,6 +279,7 @@ impl App {
             commit,
             stashmsg_popup,
             inspect_commit_popup,
+            external_editor_popup,
             help,
             revlog,
             status_tab,
@@ -299,11 +347,12 @@ impl App {
 
     fn update_commands(&mut self) {
         self.help.set_cmds(self.commands(true));
-        self.cmdbar.set_cmds(self.commands(false));
+        self.cmdbar.borrow_mut().set_cmds(self.commands(false));
     }
 
     fn process_queue(&mut self) -> Result<NeedsUpdate> {
         let mut flags = NeedsUpdate::empty();
+
         loop {
             let front = self.queue.borrow_mut().pop_front();
             if let Some(e) = front {
@@ -325,7 +374,7 @@ impl App {
         match ev {
             InternalEvent::ConfirmedAction(action) => match action {
                 Action::Reset(r) => {
-                    if Status::reset(&r) {
+                    if self.status_tab.reset(&r) {
                         flags.insert(NeedsUpdate::ALL);
                     }
                 }
@@ -334,26 +383,15 @@ impl App {
                         flags.insert(NeedsUpdate::ALL);
                     }
                 }
+                Action::ResetHunk(path, hash) => {
+                    sync::reset_hunk(CWD, path, hash)?;
+                    flags.insert(NeedsUpdate::ALL);
+                }
             },
             InternalEvent::ConfirmAction(action) => {
                 self.reset.open(action)?;
                 flags.insert(NeedsUpdate::COMMANDS);
             }
-            InternalEvent::AddHunk(hash) => {
-                if let Some((path, is_stage)) =
-                    self.status_tab.selected_path()
-                {
-                    if is_stage {
-                        if sync::unstage_hunk(CWD, path, hash)? {
-                            flags.insert(NeedsUpdate::ALL);
-                        }
-                    } else if sync::stage_hunk(CWD, path, hash)
-                        .is_ok()
-                    {
-                        flags.insert(NeedsUpdate::ALL);
-                    }
-                }
-            }
             InternalEvent::ShowErrorMsg(msg) => {
                 self.msg.show_msg(msg.as_str())?;
                 flags
@@ -370,6 +408,11 @@ impl App {
                 self.inspect_commit_popup.open(id)?;
                 flags.insert(NeedsUpdate::ALL | NeedsUpdate::COMMANDS)
             }
+            InternalEvent::SuspendPolling => {
+                self.input.set_polling(false);
+                self.external_editor_popup.show()?;
+                flags.insert(NeedsUpdate::COMMANDS)
+            }
         };
 
         Ok(flags)
@@ -423,10 +466,11 @@ impl App {
             || self.msg.is_visible()
             || self.stashmsg_popup.is_visible()
             || self.inspect_commit_popup.is_visible()
+            || self.external_editor_popup.is_visible()
     }
 
     fn draw_popups<B: Backend>(
-        &mut self,
+        &self,
         f: &mut Frame<B>,
     ) -> Result<()> {
         let size = Layout::default()
@@ -434,7 +478,7 @@ impl App {
             .constraints(
                 [
                     Constraint::Min(1),
-                    Constraint::Length(self.cmdbar.height()),
+                    Constraint::Length(self.cmdbar.borrow().height()),
                 ]
                 .as_ref(),
             )
@@ -446,27 +490,35 @@ impl App {
         self.help.draw(f, size)?;
         self.msg.draw(f, size)?;
         self.inspect_commit_popup.draw(f, size)?;
+        self.external_editor_popup.draw(f, size)?;
 
         Ok(())
     }
 
     //TODO: make this dynamic
     fn draw_tabs<B: Backend>(&self, f: &mut Frame<B>, r: Rect) {
+        let r = r.inner(&Margin {
+            vertical: 0,
+            horizontal: 1,
+        });
+
+        let tabs = &[
+            strings::TAB_STATUS,
+            strings::TAB_LOG,
+            strings::TAB_STASHING,
+            strings::TAB_STASHES,
+        ];
+
         f.render_widget(
             Tabs::default()
-                .block(Block::default().borders(Borders::BOTTOM))
-                .titles(&[
-                    strings::TAB_STATUS,
-                    strings::TAB_LOG,
-                    strings::TAB_STASHING,
-                    strings::TAB_STASHES,
-                ])
-                .style(Style::default())
-                .highlight_style(
-                    self.theme
-                        .tab(true)
-                        .modifier(Modifier::UNDERLINED),
+                .block(
+                    Block::default()
+                        .borders(Borders::BOTTOM)
+                        .border_style(self.theme.block(false)),
                 )
+                .titles(tabs)
+                .style(self.theme.tab(false))
+                .highlight_style(self.theme.tab(true))
                 .divider(strings::TAB_DIVIDER)
                 .select(self.tab),
             r,
