@@ -8,10 +8,10 @@ use crate::{
     keys,
     queue::{Action, InternalEvent, NeedsUpdate, Queue, ResetItem},
     strings,
-    ui::style::Theme,
+    ui::style::SharedTheme,
 };
 use anyhow::Result;
-use asyncgit::{sync, StatusItem, StatusItemType, CWD};
+use asyncgit::{cached, sync, StatusItem, StatusItemType, CWD};
 use crossterm::event::Event;
 use std::path::Path;
 use strings::commands;
@@ -35,9 +35,11 @@ macro_rules! try_or_popup {
 
 ///
 pub struct ChangesComponent {
+    title: String,
     files: FileTreeComponent,
     is_working_dir: bool,
     queue: Queue,
+    branch_name: cached::BranchName,
 }
 
 impl ChangesComponent {
@@ -47,9 +49,10 @@ impl ChangesComponent {
         focus: bool,
         is_working_dir: bool,
         queue: Queue,
-        theme: &Theme,
+        theme: SharedTheme,
     ) -> Self {
         Self {
+            title: title.into(),
             files: FileTreeComponent::new(
                 title,
                 focus,
@@ -58,11 +61,21 @@ impl ChangesComponent {
             ),
             is_working_dir,
             queue,
+            branch_name: cached::BranchName::new(CWD),
         }
     }
 
     ///
     pub fn update(&mut self, list: &[StatusItem]) -> Result<()> {
+        if self.is_working_dir {
+            if let Ok(branch_name) = self.branch_name.lookup() {
+                self.files.set_title(format!(
+                    "{} - {{{}}}",
+                    &self.title, branch_name,
+                ))
+            }
+        }
+
         self.files.update(list)?;
 
         Ok(())
@@ -173,7 +186,7 @@ impl ChangesComponent {
 
 impl DrawableComponent for ChangesComponent {
     fn draw<B: Backend>(
-        &mut self,
+        &self,
         f: &mut Frame<B>,
         r: Rect,
     ) -> Result<()> {
