@@ -7,19 +7,21 @@ use crate::{
     keys,
     strings::commands,
     ui::calc_scroll_top,
-    ui::style::Theme,
+    ui::style::{SharedTheme, Theme},
 };
 use anyhow::Result;
-use asyncgit::sync;
+use asyncgit::sync::Tags;
 use crossterm::event::Event;
-use std::{borrow::Cow, cmp, convert::TryFrom, time::Instant};
-use sync::Tags;
+use std::{
+    borrow::Cow, cell::Cell, cmp, convert::TryFrom, time::Instant,
+};
 use tui::{
     backend::Backend,
     layout::{Alignment, Rect},
     widgets::{Block, Borders, Paragraph, Text},
     Frame,
 };
+use unicode_width::UnicodeWidthStr;
 
 const ELEMENTS_PER_LINE: usize = 10;
 
@@ -27,27 +29,29 @@ const ELEMENTS_PER_LINE: usize = 10;
 pub struct CommitList {
     title: String,
     selection: usize,
+    branch: Option<String>,
     count_total: usize,
     items: ItemBatch,
     scroll_state: (Instant, f32),
     tags: Option<Tags>,
-    current_size: (u16, u16),
-    scroll_top: usize,
-    theme: Theme,
+    current_size: Cell<(u16, u16)>,
+    scroll_top: Cell<usize>,
+    theme: SharedTheme,
 }
 
 impl CommitList {
     ///
-    pub fn new(title: &str, theme: &Theme) -> Self {
+    pub fn new(title: &str, theme: SharedTheme) -> Self {
         Self {
             items: ItemBatch::default(),
             selection: 0,
+            branch: None,
             count_total: 0,
             scroll_state: (Instant::now(), 0_f32),
             tags: None,
-            current_size: (0, 0),
-            scroll_top: 0,
-            theme: *theme,
+            current_size: Cell::new((0, 0)),
+            scroll_top: Cell::new(0),
+            theme,
             title: String::from(title),
         }
     }
@@ -57,14 +61,19 @@ impl CommitList {
         &mut self.items
     }
 
+    ///
+    pub fn set_branch(&mut self, name: Option<String>) {
+        self.branch = name;
+    }
+
     ///
     pub const fn selection(&self) -> usize {
         self.selection
     }
 
     ///
-    pub const fn current_size(&self) -> (u16, u16) {
-        self.current_size
+    pub fn current_size(&self) -> (u16, u16) {
+        self.current_size.get()
     }
 
     ///
@@ -114,7 +123,7 @@ impl CommitList {
             usize::try_from(self.scroll_state.1 as i64)?.max(1);
 
         let page_offset =
-            usize::from(self.current_size.1).saturating_sub(1);
+            usize::from(self.current_size.get().1).saturating_sub(1);
 
         let new_selection = match scroll {
             ScrollType::Up => {
@@ -173,6 +182,7 @@ impl CommitList {
         txt: &mut Vec<Text<'a>>,
         tags: Option<String>,
         theme: &Theme,
+        width: usize,
     ) {
         txt.reserve(ELEMENTS_PER_LINE);
 
@@ -196,9 +206,13 @@ impl CommitList {
 
         txt.push(splitter.clone());
 
+        let author_width =
+            (width.saturating_sub(19) / 3).max(3).min(20);
+        let author = string_width_align(&e.author, author_width);
+
         // commit author
         txt.push(Text::Styled(
-            Cow::from(e.author.as_str()),
+            author.into(),
             theme.commit_author(selected),
         ));
 
@@ -224,7 +238,7 @@ impl CommitList {
         txt.push(Text::Raw(Cow::from("\n")));
     }
 
-    fn get_text(&self, height: usize) -> Vec<Text> {
+    fn get_text(&self, height: usize, width: usize) -> Vec<Text> {
         let selection = self.relative_selection();
 
         let mut txt = Vec::with_capacity(height * ELEMENTS_PER_LINE);
@@ -232,7 +246,7 @@ impl CommitList {
         for (idx, e) in self
             .items
             .iter()
-            .skip(self.scroll_top)
+            .skip(self.scroll_top.get())
             .take(height)
             .enumerate()
         {
@@ -246,10 +260,11 @@ impl CommitList {
 
             Self::add_entry(
                 e,
-                idx + self.scroll_top == selection,
+                idx + self.scroll_top.get() == selection,
                 &mut txt,
                 tags,
                 &self.theme,
+                width,
             );
         }
 
@@ -264,41 +279,52 @@ impl CommitList {
 
 impl DrawableComponent for CommitList {
     fn draw<B: Backend>(
-        &mut self,
+        &self,
         f: &mut Frame<B>,
         area: Rect,
     ) -> Result<()> {
-        self.current_size = (
+        let current_size = (
             area.width.saturating_sub(2),
             area.height.saturating_sub(2),
         );
+        self.current_size.set(current_size);
 
-        let height_in_lines = self.current_size.1 as usize;
+        let height_in_lines = self.current_size.get().1 as usize;
         let selection = self.relative_selection();
 
-        self.scroll_top = calc_scroll_top(
-            self.scroll_top,
+        self.scroll_top.set(calc_scroll_top(
+            self.scroll_top.get(),
             height_in_lines,
             selection,
-        );
+        ));
+
+        let branch_post_fix =
+            self.branch.as_ref().map(|b| format!("- {{{}}}", b));
 
         let title = format!(
-            "{} {}/{}",
+            "{} {}/{} {}",
             self.title,
             self.count_total.saturating_sub(self.selection),
             self.count_total,
+            branch_post_fix.as_deref().unwrap_or(""),
         );
 
         f.render_widget(
-            Paragraph::new(self.get_text(height_in_lines).iter())
-                .block(
-                    Block::default()
-                        .borders(Borders::ALL)
-                        .title(title.as_str())
-                        .border_style(self.theme.block(true))
-                        .title_style(self.theme.title(true)),
+            Paragraph::new(
+                self.get_text(
+                    height_in_lines,
+                    current_size.0 as usize,
                 )
-                .alignment(Alignment::Left),
+                .iter(),
+            )
+            .block(
+                Block::default()
+                    .borders(Borders::ALL)
+                    .title(title.as_str())
+                    .border_style(self.theme.block(true))
+                    .title_style(self.theme.title(true)),
+            )
+            .alignment(Alignment::Left),
             area,
         );
 
@@ -350,3 +376,53 @@ impl Component for CommitList {
         CommandBlocking::PassingOn
     }
 }
+
+#[inline]
+fn string_width_align(s: &str, width: usize) -> String {
+    static POSTFIX: &str = "..";
+
+    let len = UnicodeWidthStr::width(s);
+    let width_wo_postfix = width.saturating_sub(POSTFIX.len());
+
+    if (len >= width_wo_postfix && len <= width)
+        || (len <= width_wo_postfix)
+    {
+        format!("{:w$}", s, w = width)
+    } else {
+        let mut s = s.to_string();
+        s.truncate(find_truncate_point(&s, width_wo_postfix));
+        format!("{}{}", s, POSTFIX)
+    }
+}
+
+#[inline]
+fn find_truncate_point(s: &str, chars: usize) -> usize {
+    s.chars().take(chars).map(char::len_utf8).sum()
+}
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+
+    #[test]
+    fn test_string_width_align() {
+        assert_eq!(string_width_align("123", 3), "123");
+        assert_eq!(string_width_align("123", 2), "..");
+        assert_eq!(string_width_align("123", 3), "123");
+        assert_eq!(string_width_align("12345", 6), "12345 ");
+        assert_eq!(string_width_align("1234556", 4), "12..");
+    }
+
+    #[test]
+    fn test_string_width_align_unicode() {
+        assert_eq!(string_width_align("äste", 3), "ä..");
+        assert_eq!(
+            string_width_align("wüsten äste", 10),
+            "wüsten ä.."
+        );
+        assert_eq!(
+            string_width_align("Jon Grythe Stødle", 19),
+            "Jon Grythe Stødle  "
+        );
+    }
+}
