@@ -3,13 +3,13 @@ use crate::{
     components::{CommandInfo, Component},
     keys,
     queue::{Action, InternalEvent, NeedsUpdate, Queue, ResetItem},
-    strings,
-    ui::{calc_scroll_top, style::Theme},
+    strings::{self, commands},
+    ui::{calc_scroll_top, style::SharedTheme},
 };
 use asyncgit::{hash, sync, DiffLine, DiffLineType, FileDiff, CWD};
+use bytesize::ByteSize;
 use crossterm::event::Event;
-use std::{borrow::Cow, cmp, path::Path};
-use strings::commands;
+use std::{borrow::Cow, cell::Cell, cmp, path::Path};
 use tui::{
     backend::Backend,
     layout::{Alignment, Rect},
@@ -29,35 +29,38 @@ struct Current {
 
 ///
 pub struct DiffComponent {
-    diff: FileDiff,
+    diff: Option<FileDiff>,
     selection: usize,
     selected_hunk: Option<usize>,
-    current_size: (u16, u16),
+    current_size: Cell<(u16, u16)>,
     focused: bool,
     current: Current,
-    scroll_top: usize,
+    scroll_top: Cell<usize>,
     queue: Option<Queue>,
-    theme: Theme,
+    theme: SharedTheme,
 }
 
 impl DiffComponent {
     ///
-    pub fn new(queue: Option<Queue>, theme: &Theme) -> Self {
+    pub fn new(queue: Option<Queue>, theme: SharedTheme) -> Self {
         Self {
             focused: false,
             queue,
             current: Current::default(),
             selected_hunk: None,
-            diff: FileDiff::default(),
-            current_size: (0, 0),
+            diff: None,
+            current_size: Cell::new((0, 0)),
             selection: 0,
-            scroll_top: 0,
-            theme: *theme,
+            scroll_top: Cell::new(0),
+            theme,
         }
     }
     ///
-    const fn can_scroll(&self) -> bool {
-        self.diff.lines > 1
+    fn can_scroll(&self) -> bool {
+        self.diff
+            .as_ref()
+            .map(|diff| diff.lines > 1)
+            .unwrap_or_default()
     }
     ///
     pub fn current(&self) -> (String, bool) {
@@ -66,8 +69,8 @@ impl DiffComponent {
     ///
     pub fn clear(&mut self) -> Result<()> {
         self.current = Current::default();
-        self.diff = FileDiff::default();
-        self.scroll_top = 0;
+        self.diff = None;
+        self.scroll_top.set(0);
         self.selection = 0;
         self.selected_hunk = None;
 
@@ -88,12 +91,13 @@ impl DiffComponent {
                 is_stage,
                 hash,
             };
-            self.diff = diff;
-            self.scroll_top = 0;
-            self.selection = 0;
 
             self.selected_hunk =
-                Self::find_selected_hunk(&self.diff, self.selection)?;
+                Self::find_selected_hunk(&diff, self.selection)?;
+
+            self.diff = Some(diff);
+            self.scroll_top.set(0);
+            self.selection = 0;
         }
 
         Ok(())
@@ -103,30 +107,35 @@ impl DiffComponent {
         &mut self,
         move_type: ScrollType,
     ) -> Result<()> {
-        let old = self.selection;
+        if let Some(diff) = &self.diff {
+            let old = self.selection;
 
-        let max = self.diff.lines.saturating_sub(1) as usize;
+            let max = diff.lines.saturating_sub(1) as usize;
 
-        self.selection = match move_type {
-            ScrollType::Down => old.saturating_add(1),
-            ScrollType::Up => old.saturating_sub(1),
-            ScrollType::Home => 0,
-            ScrollType::End => max,
-            ScrollType::PageDown => self.selection.saturating_add(
-                self.current_size.1.saturating_sub(1) as usize,
-            ),
-            ScrollType::PageUp => self.selection.saturating_sub(
-                self.current_size.1.saturating_sub(1) as usize,
-            ),
-        };
+            self.selection = match move_type {
+                ScrollType::Down => old.saturating_add(1),
+                ScrollType::Up => old.saturating_sub(1),
+                ScrollType::Home => 0,
+                ScrollType::End => max,
+                ScrollType::PageDown => {
+                    self.selection.saturating_add(
+                        self.current_size.get().1.saturating_sub(1)
+                            as usize,
+                    )
+                }
+                ScrollType::PageUp => self.selection.saturating_sub(
+                    self.current_size.get().1.saturating_sub(1)
+                        as usize,
+                ),
+            };
 
-        self.selection = cmp::min(max, self.selection);
+            self.selection = cmp::min(max, self.selection);
 
-        if old != self.selection {
-            self.selected_hunk =
-                Self::find_selected_hunk(&self.diff, self.selection)?;
+            if old != self.selection {
+                self.selected_hunk =
+                    Self::find_selected_hunk(diff, self.selection)?;
+            }
         }
-
         Ok(())
     }
 
@@ -154,49 +163,96 @@ impl DiffComponent {
     }
 
     fn get_text(&self, width: u16, height: u16) -> Result<Vec<Text>> {
-        let selection = self.selection;
-
-        let min = self.scroll_top;
-        let max = min + height as usize;
-
         let mut res = Vec::new();
-        let mut line_cursor = 0_usize;
-        let mut lines_added = 0_usize;
+        if let Some(diff) = &self.diff {
+            if diff.hunks.is_empty() {
+                let is_positive = diff.size_delta >= 0;
+                let delta_byte_size =
+                    ByteSize::b(diff.size_delta.abs() as u64);
+                let sign = if is_positive { "+" } else { "-" };
+                res.extend(vec![
+                    Text::Raw(Cow::from("size: ")),
+                    Text::Styled(
+                        Cow::from(format!(
+                            "{}",
+                            ByteSize::b(diff.sizes.0)
+                        )),
+                        self.theme.text(false, false),
+                    ),
+                    Text::Raw(Cow::from(" -> ")),
+                    Text::Styled(
+                        Cow::from(format!(
+                            "{}",
+                            ByteSize::b(diff.sizes.1)
+                        )),
+                        self.theme.text(false, false),
+                    ),
+                    Text::Raw(Cow::from(" (")),
+                    Text::Styled(
+                        Cow::from(format!(
+                            "{}{:}",
+                            sign, delta_byte_size
+                        )),
+                        self.theme.diff_line(
+                            if is_positive {
+                                DiffLineType::Add
+                            } else {
+                                DiffLineType::Delete
+                            },
+                            false,
+                        ),
+                    ),
+                    Text::Raw(Cow::from(")")),
+                ]);
+            } else {
+                let selection = self.selection;
 
-        for (i, hunk) in self.diff.hunks.iter().enumerate() {
-            let hunk_selected =
-                self.selected_hunk.map_or(false, |s| s == i);
+                let min = self.scroll_top.get();
+                let max = min + height as usize;
 
-            if lines_added >= height as usize {
-                break;
-            }
+                let mut line_cursor = 0_usize;
+                let mut lines_added = 0_usize;
 
-            let hunk_len = hunk.lines.len();
-            let hunk_min = line_cursor;
-            let hunk_max = line_cursor + hunk_len;
+                for (i, hunk) in diff.hunks.iter().enumerate() {
+                    let hunk_selected =
+                        self.selected_hunk.map_or(false, |s| s == i);
 
-            if Self::hunk_visible(hunk_min, hunk_max, min, max) {
-                for (i, line) in hunk.lines.iter().enumerate() {
-                    if line_cursor >= min && line_cursor <= max {
-                        Self::add_line(
-                            &mut res,
-                            width,
-                            line,
-                            selection == line_cursor,
-                            hunk_selected,
-                            i == hunk_len as usize - 1,
-                            self.theme,
-                        );
-                        lines_added += 1;
+                    if lines_added >= height as usize {
+                        break;
                     }
 
-                    line_cursor += 1;
+                    let hunk_len = hunk.lines.len();
+                    let hunk_min = line_cursor;
+                    let hunk_max = line_cursor + hunk_len;
+
+                    if Self::hunk_visible(
+                        hunk_min, hunk_max, min, max,
+                    ) {
+                        for (i, line) in hunk.lines.iter().enumerate()
+                        {
+                            if line_cursor >= min
+                                && line_cursor <= max
+                            {
+                                Self::add_line(
+                                    &mut res,
+                                    width,
+                                    line,
+                                    selection == line_cursor,
+                                    hunk_selected,
+                                    i == hunk_len as usize - 1,
+                                    &self.theme,
+                                );
+                                lines_added += 1;
+                            }
+
+                            line_cursor += 1;
+                        }
+                    } else {
+                        line_cursor += hunk_len;
+                    }
                 }
-            } else {
-                line_cursor += hunk_len;
             }
         }
-
         Ok(res)
     }
 
@@ -207,7 +263,7 @@ impl DiffComponent {
         selected: bool,
         selected_hunk: bool,
         end_of_hunk: bool,
-        theme: Theme,
+        theme: &SharedTheme,
     ) {
         {
             let style = theme.diff_hunk_marker(selected_hunk);
@@ -272,26 +328,34 @@ impl DiffComponent {
     }
 
     fn unstage_hunk(&mut self) -> Result<()> {
-        if let Some(hunk) = self.selected_hunk {
-            let hash = self.diff.hunks[hunk].header_hash;
-            sync::unstage_hunk(CWD, self.current.path.clone(), hash)?;
-            self.queue_update();
+        if let Some(diff) = &self.diff {
+            if let Some(hunk) = self.selected_hunk {
+                let hash = diff.hunks[hunk].header_hash;
+                sync::unstage_hunk(
+                    CWD,
+                    self.current.path.clone(),
+                    hash,
+                )?;
+                self.queue_update();
+            }
         }
 
         Ok(())
     }
 
     fn stage_hunk(&mut self) -> Result<()> {
-        if let Some(hunk) = self.selected_hunk {
-            let path = self.current.path.clone();
-            if self.diff.untracked {
-                sync::stage_add_file(CWD, Path::new(&path))?;
-            } else {
-                let hash = self.diff.hunks[hunk].header_hash;
-                sync::stage_hunk(CWD, path, hash)?;
-            }
+        if let Some(diff) = &self.diff {
+            if let Some(hunk) = self.selected_hunk {
+                let path = self.current.path.clone();
+                if diff.untracked {
+                    sync::stage_add_file(CWD, Path::new(&path))?;
+                } else {
+                    let hash = diff.hunks[hunk].header_hash;
+                    sync::stage_hunk(CWD, path, hash)?;
+                }
 
-            self.queue_update();
+                self.queue_update();
+            }
         }
 
         Ok(())
@@ -306,21 +370,22 @@ impl DiffComponent {
     }
 
     fn reset_hunk(&self) -> Result<()> {
-        if let Some(hunk) = self.selected_hunk {
-            let hash = self.diff.hunks[hunk].header_hash;
+        if let Some(diff) = &self.diff {
+            if let Some(hunk) = self.selected_hunk {
+                let hash = diff.hunks[hunk].header_hash;
 
-            self.queue
-                .as_ref()
-                .expect("try using queue in immutable diff")
-                .borrow_mut()
-                .push_back(InternalEvent::ConfirmAction(
-                    Action::ResetHunk(
-                        self.current.path.clone(),
-                        hash,
-                    ),
-                ));
+                self.queue
+                    .as_ref()
+                    .expect("try using queue in immutable diff")
+                    .borrow_mut()
+                    .push_back(InternalEvent::ConfirmAction(
+                        Action::ResetHunk(
+                            self.current.path.clone(),
+                            hash,
+                        ),
+                    ));
+            }
         }
-
         Ok(())
     }
 
@@ -350,24 +415,27 @@ impl DiffComponent {
 
 impl DrawableComponent for DiffComponent {
     fn draw<B: Backend>(
-        &mut self,
+        &self,
         f: &mut Frame<B>,
         r: Rect,
     ) -> Result<()> {
-        self.current_size =
-            (r.width.saturating_sub(2), r.height.saturating_sub(2));
+        self.current_size.set((
+            r.width.saturating_sub(2),
+            r.height.saturating_sub(2),
+        ));
 
-        self.scroll_top = calc_scroll_top(
-            self.scroll_top,
-            self.current_size.1 as usize,
+        self.scroll_top.set(calc_scroll_top(
+            self.scroll_top.get(),
+            self.current_size.get().1 as usize,
             self.selection,
-        );
+        ));
 
         let title =
             format!("{}{}", strings::TITLE_DIFF, self.current.path);
         f.render_widget(
             Paragraph::new(
-                self.get_text(r.width, self.current_size.1)?.iter(),
+                self.get_text(r.width, self.current_size.get().1)?
+                    .iter(),
             )
             .block(
                 Block::default()
@@ -466,10 +534,12 @@ impl Component for DiffComponent {
                         if !self.is_immutable()
                             && !self.is_stage() =>
                     {
-                        if self.diff.untracked {
-                            self.reset_untracked()?;
-                        } else {
-                            self.reset_hunk()?;
+                        if let Some(diff) = &self.diff {
+                            if diff.untracked {
+                                self.reset_untracked()?;
+                            } else {
+                                self.reset_hunk()?;
+                            }
                         }
                         Ok(true)
                     }
@@ -506,7 +576,7 @@ mod tests {
             false,
             false,
             false,
-            Theme::default(),
+            &SharedTheme::default(),
         );
 
         assert_eq!(text.len(), 2);
