@@ -2,33 +2,38 @@ use super::{
     visibility_blocking, CommandBlocking, CommandInfo, Component,
     DrawableComponent,
 };
-use crate::{keys, strings, ui, ui::style::Theme, version::Version};
+use crate::{
+    keys,
+    strings::{self, commands},
+    ui,
+    version::Version,
+};
 use asyncgit::hash;
 use crossterm::event::Event;
 use itertools::Itertools;
 use std::{borrow::Cow, cmp, convert::TryFrom};
-use strings::commands;
 use tui::{
     backend::Backend,
     layout::{Alignment, Constraint, Direction, Layout, Rect},
     style::{Modifier, Style},
-    widgets::{Block, Borders, Clear, Paragraph, Text},
+    widgets::{Block, BorderType, Borders, Clear, Paragraph, Text},
     Frame,
 };
 
 use anyhow::Result;
+use ui::style::SharedTheme;
 
 ///
 pub struct HelpComponent {
     cmds: Vec<CommandInfo>,
     visible: bool,
     selection: u16,
-    theme: Theme,
+    theme: SharedTheme,
 }
 
 impl DrawableComponent for HelpComponent {
     fn draw<B: Backend>(
-        &mut self,
+        &self,
         f: &mut Frame<B>,
         _rect: Rect,
     ) -> Result<()> {
@@ -45,7 +50,8 @@ impl DrawableComponent for HelpComponent {
             f.render_widget(
                 Block::default()
                     .title(strings::HELP_TITLE)
-                    .borders(Borders::ALL),
+                    .borders(Borders::ALL)
+                    .border_type(BorderType::Thick),
                 area,
             );
 
@@ -97,26 +103,22 @@ impl Component for HelpComponent {
             out.clear();
         }
 
-        out.push(
-            CommandInfo::new(
-                commands::HELP_OPEN,
+        if self.visible {
+            out.push(CommandInfo::new(commands::SCROLL, true, true));
+
+            out.push(CommandInfo::new(
+                commands::CLOSE_POPUP,
                 true,
-                !self.visible,
-            )
-            .order(99),
-        );
+                true,
+            ));
+        }
 
-        out.push(CommandInfo::new(
-            commands::SCROLL,
-            true,
-            self.visible,
-        ));
-
-        out.push(CommandInfo::new(
-            commands::CLOSE_POPUP,
-            true,
-            self.visible,
-        ));
+        if !self.visible || force_all {
+            out.push(
+                CommandInfo::new(commands::HELP_OPEN, true, true)
+                    .order(99),
+            );
+        }
 
         visibility_blocking(self)
     }
@@ -157,12 +159,12 @@ impl Component for HelpComponent {
 }
 
 impl HelpComponent {
-    pub fn new(theme: &Theme) -> Self {
+    pub const fn new(theme: SharedTheme) -> Self {
         Self {
             cmds: vec![],
             visible: false,
             selection: 0,
-            theme: *theme,
+            theme,
         }
     }
     ///
@@ -193,7 +195,7 @@ impl HelpComponent {
         }
     }
 
-    fn get_text<'a>(&self) -> Vec<Text<'a>> {
+    fn get_text(&self) -> Vec<Text> {
         let mut txt = Vec::new();
 
         let mut processed = 0_u16;
@@ -202,7 +204,7 @@ impl HelpComponent {
             &self.cmds.iter().group_by(|e| e.text.group)
         {
             txt.push(Text::Styled(
-                Cow::from(format!(" {}\n", key)),
+                Cow::from(format!("{}\n", key)),
                 Style::default().modifier(Modifier::REVERSED),
             ));
 
