@@ -3,12 +3,11 @@ use crate::{
         popup_paragraph, visibility_blocking, CommandBlocking,
         CommandInfo, Component, DrawableComponent,
     },
-    strings, ui,
-    ui::style::Theme,
+    strings::commands,
+    ui::{self, style::SharedTheme},
 };
 use anyhow::Result;
 use crossterm::event::{Event, KeyCode, KeyModifiers};
-use strings::commands;
 use tui::{
     backend::Backend,
     layout::Rect,
@@ -23,21 +22,21 @@ pub struct TextInputComponent {
     default_msg: String,
     msg: String,
     visible: bool,
-    theme: Theme,
+    theme: SharedTheme,
     cursor_position: usize,
 }
 
 impl TextInputComponent {
     ///
     pub fn new(
-        theme: &Theme,
+        theme: SharedTheme,
         title: &str,
         default_msg: &str,
     ) -> Self {
         Self {
             msg: String::default(),
             visible: false,
-            theme: *theme,
+            theme,
             title: title.to_string(),
             default_msg: default_msg.to_string(),
             cursor_position: 0,
@@ -90,6 +89,7 @@ impl TextInputComponent {
     /// Set the `msg`.
     pub fn set_text(&mut self, msg: String) {
         self.msg = msg;
+        self.cursor_position = 0;
     }
 
     /// Set the `title`.
@@ -100,7 +100,7 @@ impl TextInputComponent {
 
 impl DrawableComponent for TextInputComponent {
     fn draw<B: Backend>(
-        &mut self,
+        &self,
         f: &mut Frame<B>,
         _rect: Rect,
     ) -> Result<()> {
