@@ -12,8 +12,10 @@
 mod app;
 mod cmdbar;
 mod components;
+mod input;
 mod keys;
-mod poll;
+mod notify_mutex;
+mod profiler;
 mod queue;
 mod spinner;
 mod strings;
@@ -21,7 +23,7 @@ mod tabs;
 mod ui;
 mod version;
 
-use crate::{app::App, poll::QueueEvent};
+use crate::app::App;
 use anyhow::{anyhow, Result};
 use asyncgit::AsyncNotification;
 use backtrace::Backtrace;
@@ -37,6 +39,8 @@ use crossterm::{
     },
     ExecutableCommand,
 };
+use input::{Input, InputEvent, InputState};
+use profiler::Profiler;
 use scopeguard::defer;
 use scopetime::scope_time;
 use simplelog::{Config, LevelFilter, WriteLogger};
@@ -56,16 +60,30 @@ use tui::{
 };
 
 static TICK_INTERVAL: Duration = Duration::from_secs(5);
-static SPINNER_INTERVAL: Duration = Duration::from_millis(50);
+static SPINNER_INTERVAL: Duration = Duration::from_millis(80);
+
+///
+#[derive(Clone, Copy)]
+pub enum QueueEvent {
+    Tick,
+    SpinnerUpdate,
+    GitEvent(AsyncNotification),
+    InputEvent(InputEvent),
+}
 
 fn main() -> Result<()> {
     process_cmdline()?;
 
+    let _profiler = Profiler::new();
+
     if !valid_path()? {
         eprintln!("invalid path\nplease run gitui inside of a non-bare git repository");
         return Ok(());
     }
 
+    // TODO: To be removed in a future version, when upgrading from 0.6.x or earlier is unlikely
+    migrate_config()?;
+
     setup_terminal()?;
     defer! {
         shutdown_terminal().expect("shutdown failed");
@@ -77,47 +95,57 @@ fn main() -> Result<()> {
 
     let (tx_git, rx_git) = unbounded();
 
-    let mut app = App::new(&tx_git);
+    let input = Input::new();
 
-    let rx_input = poll::start_polling_thread();
+    let rx_input = input.receiver();
     let ticker = tick(TICK_INTERVAL);
     let spinner_ticker = tick(SPINNER_INTERVAL);
 
-    app.update()?;
-    draw(&mut terminal, &mut app)?;
+    let mut app = App::new(&tx_git, input);
 
     let mut spinner = Spinner::default();
+    let mut first_update = true;
 
     loop {
-        let events: Vec<QueueEvent> = select_event(
-            &rx_input,
-            &rx_git,
-            &ticker,
-            &spinner_ticker,
-        )?;
+        let event = if first_update {
+            first_update = false;
+            QueueEvent::Tick
+        } else {
+            select_event(
+                &rx_input,
+                &rx_git,
+                &ticker,
+                &spinner_ticker,
+            )?
+        };
 
         {
+            if let QueueEvent::SpinnerUpdate = event {
+                spinner.update();
+                spinner.draw(&mut terminal)?;
+                continue;
+            }
+
             scope_time!("loop");
 
-            let mut needs_draw = true;
-
-            for e in events {
-                match e {
-                    QueueEvent::InputEvent(ev) => app.event(ev)?,
-                    QueueEvent::Tick => app.update()?,
-                    QueueEvent::GitEvent(ev) => app.update_git(ev)?,
-                    QueueEvent::SpinnerUpdate => {
-                        needs_draw = false;
-                        spinner.update()
+            match event {
+                QueueEvent::InputEvent(ev) => {
+                    if let InputEvent::State(InputState::Polling) = ev
+                    {
+                        //Note: external ed closed, we need to re-hide cursor
+                        terminal.hide_cursor()?;
                     }
+                    app.event(ev)?
                 }
+                QueueEvent::Tick => app.update()?,
+                QueueEvent::GitEvent(ev) => app.update_git(ev)?,
+                QueueEvent::SpinnerUpdate => unreachable!(),
             }
 
-            if needs_draw {
-                draw(&mut terminal, &mut app)?;
-            }
+            draw(&mut terminal, &app)?;
 
-            spinner.draw(&mut terminal, app.any_work_pending())?;
+            spinner.set_state(app.any_work_pending());
+            spinner.draw(&mut terminal)?;
 
             if app.is_quit() {
                 break;
@@ -142,8 +170,12 @@ fn shutdown_terminal() -> Result<()> {
 
 fn draw<B: Backend>(
     terminal: &mut Terminal<B>,
-    app: &mut App,
+    app: &App,
 ) -> io::Result<()> {
+    if app.requires_redraw() {
+        terminal.resize(terminal.size()?)?;
+    }
+
     terminal.draw(|mut f| {
         if let Err(e) = app.draw(&mut f) {
             log::error!("failed to draw: {:?}", e)
@@ -157,13 +189,11 @@ fn valid_path() -> Result<bool> {
 }
 
 fn select_event(
-    rx_input: &Receiver<Vec<QueueEvent>>,
+    rx_input: &Receiver<InputEvent>,
     rx_git: &Receiver<AsyncNotification>,
     rx_ticker: &Receiver<Instant>,
     rx_spinner: &Receiver<Instant>,
-) -> Result<Vec<QueueEvent>> {
-    let mut events: Vec<QueueEvent> = Vec::new();
-
+) -> Result<QueueEvent> {
     let mut sel = Select::new();
 
     sel.recv(rx_input);
@@ -174,21 +204,15 @@ fn select_event(
     let oper = sel.select();
     let index = oper.index();
 
-    match index {
-        0 => oper.recv(rx_input).map(|inputs| events.extend(inputs)),
-        1 => oper
-            .recv(rx_git)
-            .map(|ev| events.push(QueueEvent::GitEvent(ev))),
-        2 => oper
-            .recv(rx_ticker)
-            .map(|_| events.push(QueueEvent::Tick)),
-        3 => oper
-            .recv(rx_spinner)
-            .map(|_| events.push(QueueEvent::SpinnerUpdate)),
+    let ev = match index {
+        0 => oper.recv(rx_input).map(QueueEvent::InputEvent),
+        1 => oper.recv(rx_git).map(QueueEvent::GitEvent),
+        2 => oper.recv(rx_ticker).map(|_| QueueEvent::Tick),
+        3 => oper.recv(rx_spinner).map(|_| QueueEvent::SpinnerUpdate),
         _ => return Err(anyhow!("unknown select source")),
     }?;
 
-    Ok(events)
+    Ok(ev)
 }
 
 fn start_terminal<W: Write>(
@@ -202,7 +226,24 @@ fn start_terminal<W: Write>(
     Ok(terminal)
 }
 
-fn get_app_config_path() -> Result<PathBuf> {
+fn migrate_config() -> Result<()> {
+    let cache_path: PathBuf = get_app_cache_path()?;
+
+    let entries = cache_path.read_dir()?.flatten().filter(|entry| {
+        !entry.file_name().to_string_lossy().ends_with(".log")
+    });
+
+    let config_path: PathBuf = get_app_config_path()?;
+    for entry in entries {
+        let mut config_path: PathBuf = config_path.clone();
+        config_path.push(entry.file_name());
+        fs::rename(entry.path(), config_path)?;
+    }
+
+    Ok(())
+}
+
+fn get_app_cache_path() -> Result<PathBuf> {
     let mut path = dirs::cache_dir()
         .ok_or_else(|| anyhow!("failed to find os cache dir."))?;
 
@@ -211,8 +252,17 @@ fn get_app_config_path() -> Result<PathBuf> {
     Ok(path)
 }
 
+fn get_app_config_path() -> Result<PathBuf> {
+    let mut path = dirs::config_dir()
+        .ok_or_else(|| anyhow!("failed to find os config dir."))?;
+
+    path.push("gitui");
+    fs::create_dir_all(&path)?;
+    Ok(path)
+}
+
 fn setup_logging() -> Result<()> {
-    let mut path = get_app_config_path()?;
+    let mut path = get_app_cache_path()?;
     path.push("gitui.log");
 
     let _ = WriteLogger::init(
