@@ -26,6 +26,7 @@ pub struct ResetItem {
 ///
 pub enum Action {
     Reset(ResetItem),
+    ResetHunk(String, u64),
     StashDrop(CommitId),
 }
 
@@ -36,8 +37,6 @@ pub enum InternalEvent {
     ///
     ConfirmedAction(Action),
     ///
-    AddHunk(u64),
-    ///
     ShowErrorMsg(String),
     ///
     Update(NeedsUpdate),
@@ -49,6 +48,10 @@ pub enum InternalEvent {
     TabSwitch,
     ///
     InspectCommit(CommitId),
+    ///
+    //TODO: make this a generic OpenExternalEditor to also use it for other places
+    //(see https://github.com/extrawurst/gitui/issues/166)
+    SuspendPolling,
 }
 
 ///
