@@ -6,14 +6,18 @@ pub static TAB_STATUS: &str = "Status [1]";
 pub static TAB_LOG: &str = "Log [2]";
 pub static TAB_STASHING: &str = "Stashing [3]";
 pub static TAB_STASHES: &str = "Stashes [4]";
-pub static TAB_DIVIDER: &str = "  |  ";
+pub static TAB_DIVIDER: &str = " | ";
 
 pub static CMD_SPLITTER: &str = " ";
 
+pub static MSG_OPENING_EDITOR: &str = "opening editor...";
 pub static MSG_TITLE_ERROR: &str = "Error";
 pub static COMMIT_TITLE: &str = "Commit";
 pub static COMMIT_TITLE_AMEND: &str = "Commit (Amend)";
 pub static COMMIT_MSG: &str = "type commit message..";
+pub static COMMIT_EDITOR_MSG: &str = r##"
+# Edit your commit message
+# Lines starting with '#' will be ignored"##;
 pub static STASH_POPUP_TITLE: &str = "Stash";
 pub static STASH_POPUP_MSG: &str = "type name (optional)";
 pub static CONFIRM_TITLE_RESET: &str = "Reset";
@@ -50,13 +54,13 @@ pub mod order {
 pub mod commands {
     use crate::components::CommandText;
 
-    static CMD_GROUP_GENERAL: &str = "General";
-    static CMD_GROUP_DIFF: &str = "Diff";
-    static CMD_GROUP_CHANGES: &str = "Changes";
-    static CMD_GROUP_COMMIT: &str = "Commit";
-    static CMD_GROUP_STASHING: &str = "Stashing";
-    static CMD_GROUP_STASHES: &str = "Stashes";
-    static CMD_GROUP_LOG: &str = "Log";
+    static CMD_GROUP_GENERAL: &str = "-- General --";
+    static CMD_GROUP_DIFF: &str = "-- Diff --";
+    static CMD_GROUP_CHANGES: &str = "-- Changes --";
+    static CMD_GROUP_COMMIT: &str = "-- Commit --";
+    static CMD_GROUP_STASHING: &str = "-- Stashing --";
+    static CMD_GROUP_STASHES: &str = "-- Stashes --";
+    static CMD_GROUP_LOG: &str = "-- Log --";
 
     ///
     pub static TOGGLE_TABS: CommandText = CommandText::new(
@@ -90,7 +94,7 @@ pub mod commands {
     );
     ///
     pub static DIFF_HOME_END: CommandText = CommandText::new(
-        "Jump up/down [home,end,\u{11014}up,\u{2191}down]",
+        "Jump up/down [home,end,\u{2191} up,\u{2193} down]",
         "scroll to top or bottom of diff",
         CMD_GROUP_DIFF,
     );
@@ -150,6 +154,12 @@ pub mod commands {
         CMD_GROUP_COMMIT,
     );
     ///
+    pub static COMMIT_OPEN_EDITOR: CommandText = CommandText::new(
+        "Open editor [^e]",
+        "open commit editor (available in non-empty stage)",
+        CMD_GROUP_COMMIT,
+    );
+    ///
     pub static COMMIT_ENTER: CommandText = CommandText::new(
         "Commit [enter]",
         "commit (available when commit message is non-empty)",
