@@ -1,14 +1,14 @@
 use crate::{
     accessors,
     components::{
-        self, event_pump, ChangesComponent, CommandBlocking,
-        CommandInfo, Component, DiffComponent, DrawableComponent,
-        FileTreeItemKind,
+        command_pump, event_pump, visibility_blocking,
+        ChangesComponent, CommandBlocking, CommandInfo, Component,
+        DiffComponent, DrawableComponent, FileTreeItemKind,
     },
     keys,
     queue::{InternalEvent, Queue, ResetItem},
-    strings,
-    ui::style::Theme,
+    strings::{self, commands, order},
+    ui::style::SharedTheme,
 };
 use anyhow::Result;
 use asyncgit::{
@@ -16,10 +16,8 @@ use asyncgit::{
     AsyncDiff, AsyncNotification, AsyncStatus, DiffParams, DiffType,
     StatusParams, CWD,
 };
-use components::{command_pump, visibility_blocking};
 use crossbeam_channel::Sender;
 use crossterm::event::Event;
-use strings::{commands, order};
 use tui::layout::{Constraint, Direction, Layout};
 
 ///
@@ -52,7 +50,7 @@ pub struct Status {
 
 impl DrawableComponent for Status {
     fn draw<B: tui::backend::Backend>(
-        &mut self,
+        &self,
         f: &mut tui::Frame<B>,
         rect: tui::layout::Rect,
     ) -> Result<()> {
@@ -105,9 +103,9 @@ impl Status {
 
     ///
     pub fn new(
-        sender: &Sender<AsyncNotification>,
         queue: &Queue,
-        theme: &Theme,
+        sender: &Sender<AsyncNotification>,
+        theme: SharedTheme,
     ) -> Self {
         Self {
             queue: queue.clone(),
@@ -119,14 +117,14 @@ impl Status {
                 true,
                 true,
                 queue.clone(),
-                theme,
+                theme.clone(),
             ),
             index: ChangesComponent::new(
                 strings::TITLE_INDEX,
                 false,
                 false,
                 queue.clone(),
-                theme,
+                theme.clone(),
             ),
             diff: DiffComponent::new(Some(queue.clone()), theme),
             git_diff: AsyncDiff::new(sender.clone()),
