@@ -6,14 +6,17 @@ use ron::{
     ser::{to_string_pretty, PrettyConfig},
 };
 use serde::{Deserialize, Serialize};
-use std::path::PathBuf;
 use std::{
     fs::File,
     io::{Read, Write},
+    path::PathBuf,
+    rc::Rc,
 };
 use tui::style::{Color, Modifier, Style};
 
-#[derive(Serialize, Deserialize, Debug, Clone, Copy)]
+pub type SharedTheme = Rc<Theme>;
+
+#[derive(Serialize, Deserialize, Debug)]
 pub struct Theme {
     #[serde(with = "ColorDef")]
     selected_tab: Color,
@@ -22,6 +25,8 @@ pub struct Theme {
     #[serde(with = "ColorDef")]
     selection_bg: Color,
     #[serde(with = "ColorDef")]
+    cmdbar_extra_lines_bg: Color,
+    #[serde(with = "ColorDef")]
     disabled_fg: Color,
     #[serde(with = "ColorDef")]
     diff_line_add: Color,
@@ -64,12 +69,23 @@ impl Theme {
 
     pub fn tab(&self, selected: bool) -> Style {
         if selected {
-            Style::default().fg(self.selected_tab)
+            self.text(true, false).modifier(Modifier::UNDERLINED)
         } else {
-            Style::default()
+            self.text(false, false)
         }
     }
 
+    pub fn tags(&self, selected: bool) -> Style {
+        Style::default()
+            .fg(self.selected_tab)
+            .modifier(Modifier::BOLD)
+            .bg(if selected {
+                self.selection_bg
+            } else {
+                Color::Reset
+            })
+    }
+
     pub fn text(&self, enabled: bool, selected: bool) -> Style {
         match (enabled, selected) {
             (false, _) => Style::default().fg(self.disabled_fg),
@@ -153,13 +169,17 @@ impl Theme {
         Style::default().fg(self.danger_fg)
     }
 
-    pub fn toolbar(&self, enabled: bool) -> Style {
+    pub fn commandbar(&self, enabled: bool, line: usize) -> Style {
         if enabled {
             Style::default().fg(self.command_fg)
         } else {
             Style::default().fg(self.disabled_fg)
         }
-        .bg(self.selection_bg)
+        .bg(if line == 0 {
+            self.selection_bg
+        } else {
+            self.cmdbar_extra_lines_bg
+        })
     }
 
     pub fn commit_hash(&self, selected: bool) -> Style {
@@ -194,19 +214,19 @@ impl Theme {
         Ok(app_home.join("theme.ron"))
     }
 
-    fn read_file(theme_file: PathBuf) -> Result<Theme> {
+    fn read_file(theme_file: PathBuf) -> Result<Self> {
         let mut f = File::open(theme_file)?;
         let mut buffer = Vec::new();
         f.read_to_end(&mut buffer)?;
         Ok(from_bytes(&buffer)?)
     }
 
-    fn init_internal() -> Result<Theme> {
-        let file = Theme::get_theme_file()?;
+    fn init_internal() -> Result<Self> {
+        let file = Self::get_theme_file()?;
         if file.exists() {
-            Ok(Theme::read_file(file)?)
+            Ok(Self::read_file(file)?)
         } else {
-            let def = Theme::default();
+            let def = Self::default();
             if def.save().is_err() {
                 log::warn!("failed to store default theme to disk.")
             }
@@ -214,8 +234,8 @@ impl Theme {
         }
     }
 
-    pub fn init() -> Theme {
-        Theme::init_internal().unwrap_or_default()
+    pub fn init() -> Self {
+        Self::init_internal().unwrap_or_default()
     }
 }
 
@@ -224,7 +244,8 @@ impl Default for Theme {
         Self {
             selected_tab: Color::Yellow,
             command_fg: Color::White,
-            selection_bg: Color::Rgb(0, 0, 100),
+            selection_bg: Color::Blue,
+            cmdbar_extra_lines_bg: Color::Blue,
             disabled_fg: Color::DarkGray,
             diff_line_add: Color::Green,
             diff_line_delete: Color::Red,
@@ -233,7 +254,7 @@ impl Default for Theme {
             diff_file_moved: Color::LightMagenta,
             diff_file_modified: Color::Yellow,
             commit_hash: Color::Magenta,
-            commit_time: Color::Rgb(110, 110, 255),
+            commit_time: Color::LightCyan,
             commit_author: Color::Green,
             danger_fg: Color::Red,
         }
